@@ -11,133 +11,57 @@
 #define _BIRD_FILTER_DATA_H_
 
 #include "nest/bird.h"
-#include "nest/route.h"
-
-/* Type numbers must be in 0..0xff range */
-#define T_MASK 0xff
-
-/* Internal types */
-enum f_type {
-/* Nothing. Simply nothing. */
-  T_VOID = 0,
-
-  T_NONE = 1,		/* Special hack to represent missing arguments */
-
-/* User visible types, which fit in int */
-  T_INT = 0x10,
-  T_BOOL = 0x11,
-  T_PAIR = 0x12,  /*	Notice that pair is stored as integer: first << 16 | second */
-  T_QUAD = 0x13,
-
-/* Put enumerational types in 0x30..0x3f range */
-  T_ENUM_LO = 0x30,
-  T_ENUM_HI = 0x3f,
-
-  T_ENUM_RTS = 0x30,
-  T_ENUM_BGP_ORIGIN = 0x31,
-  T_ENUM_SCOPE = 0x32,
-  T_ENUM_RTD = 0x33,
-  T_ENUM_ROA = 0x34,
-  T_ENUM_ASPA = 0x35,
-  T_ENUM_NET_TYPE = 0x36,
-  T_ENUM_RA_PREFERENCE = 0x37,
-  T_ENUM_AF = 0x38,
-  T_ENUM_MPLS_POLICY = 0x39,
-
-/* new enums go here */
-  T_ENUM_EMPTY = 0x3f,	/* Special hack for atomic_aggr */
-
-#define T_ENUM T_ENUM_LO ... T_ENUM_HI
-
-/* Bigger ones */
-  T_IP = 0x20,
-  T_NET = 0x21,
-  T_STRING = 0x22,
-  T_PATH_MASK = 0x23,	/* mask for BGP path */
-  T_PATH = 0x24,		/* BGP path */
-  T_CLIST = 0x25,		/* Community list */
-  T_EC = 0x26,		/* Extended community value, u64 */
-  T_ECLIST = 0x27,		/* Extended community list */
-  T_LC = 0x28,		/* Large community value, lcomm */
-  T_LCLIST = 0x29,		/* Large community list */
-  T_RD = 0x2a,		/* Route distinguisher for VPN addresses */
-  T_PATH_MASK_ITEM = 0x2b,	/* Path mask item for path mask constructors */
-  T_BYTESTRING = 0x2c,
-
-  T_ROUTE = 0x78,
-  T_ROUTES_BLOCK = 0x79,
-  T_SET = 0x80,
-  T_PREFIX_SET = 0x81,
-} PACKED;
+#include "lib/type.h"
+#include "nest/iface.h"
 
 struct f_method {
   struct symbol *sym;
   struct f_inst *(*new_inst)(struct f_inst *obj, struct f_inst *args);
   const struct f_method *next;
   uint arg_num;
-  enum f_type args_type[];
+  enum btype args_type[];
 };
 
 /* Filter value; size of this affects filter memory consumption */
 struct f_val {
-  enum f_type type;	/* T_*  */
-  union {
-    uint i;
-    u64 ec;
-    lcomm lc;
-    vpn_rd rd;
-    ip_addr ip;
-    const net_addr *net;
-    const char *s;
-    const struct adata *bs;
-    const struct f_tree *t;
-    const struct f_trie *ti;
-    const struct adata *ad;
-    const struct f_path_mask *path_mask;
-    struct f_path_mask_item pmi;
-    struct rte *rte;
-  } val;
+  btype type;	/* T_*  */
+  union bval_long val;
 };
 
-/* Dynamic attribute definition (eattrs) */
-struct f_dynamic_attr {
-  u8 type;		/* EA type (EAF_*) */
-  u8 bit;		/* For bitfield accessors */
-  enum f_type f_type;	/* Filter type */
-  uint ea_code;		/* EA code */
-  uint flags;
-};
+#define fputip(a)   ({ ip_addr *ax = falloc(sizeof(*ax)); *ax = (a); ax; })
 
 enum f_sa_code {
-  SA_FROM = 1,
-  SA_GW,
+  SA_GW = 1,
   SA_NET,
   SA_PROTO,
-  SA_SOURCE,
-  SA_SCOPE,
   SA_DEST,
   SA_IFNAME,
   SA_IFINDEX,
   SA_WEIGHT,
-  SA_PREF,
   SA_GW_MPLS,
-  SA_GW_MPLS_STACK,
-  SA_ONLINK,
 } PACKED;
 
 /* Static attribute definition (members of struct rta) */
 struct f_static_attr {
-  enum f_type f_type;		/* Filter type */
+  btype type;			/* Data type */
   enum f_sa_code sa_code;	/* Static attribute id */
-  int readonly:1;			/* Don't allow writing */
+  int readonly:1;		/* Don't allow writing */
 };
 
+struct f_attr_bit {
+  const struct ea_class *class;
+  uint bit;
+};
+
+#define f_new_dynamic_attr_bit(_bit, _name)  ((struct f_attr_bit) { .bit = _bit, .class = ea_class_find(_name) })
+
 /* Filter l-value type */
 enum f_lval_type {
+  F_LVAL_CONSTANT,
   F_LVAL_VARIABLE,
-  F_LVAL_PREFERENCE,
   F_LVAL_SA,
   F_LVAL_EA,
+  F_LVAL_ATTR_BIT,
 };
 
 /* Filter l-value */
@@ -146,8 +70,9 @@ struct f_lval {
   struct f_inst *rte;
   union {
     struct symbol *sym;
-    struct f_dynamic_attr da;
+    const struct ea_class *da;
     struct f_static_attr sa;
+    struct f_attr_bit fab;
   };
 };
 
@@ -295,9 +220,9 @@ trie_match_next_longest_ip6(net_addr_ip6 *n, ip6_addr *found)
 
 #define F_CMP_ERROR 999
 
-const char *f_type_name(enum f_type t);
-enum f_type f_type_element_type(enum f_type t);
-struct sym_scope *f_type_method_scope(enum f_type t);
+const char *f_type_name(btype t);
+enum btype f_type_element_type(btype t);
+struct sym_scope *f_type_method_scope(btype t);
 
 int val_same(const struct f_val *v1, const struct f_val *v2);
 int val_compare(const struct f_val *v1, const struct f_val *v2);
@@ -305,6 +230,11 @@ void val_format(const struct f_val *v, buffer *buf);
 char *val_format_str(struct linpool *lp, const struct f_val *v);
 const char *val_dump(const struct f_val *v);
 
+uint val_hash(struct f_val *);
+void mem_hash_mix_f_val(u64 *, struct f_val *);
+
+struct f_val *lp_val_copy(struct linpool *lp, const struct f_val *v);
+
 static inline int val_is_ip4(const struct f_val *v)
 { return (v->type == T_IP) && ipa_is_ip4(v->val.ip); }
 int val_in_range(const struct f_val *v1, const struct f_val *v2);
@@ -336,25 +266,23 @@ val_is_undefined(struct f_val v)
     (v.val.ad == &null_adata);
 }
 
-static inline struct f_val
-val_empty(enum f_type t)
+extern const struct f_val f_const_empty_prefix_set;
+static inline struct f_val f_get_empty(btype t)
 {
-  switch (t)
-  {
-  case T_PATH:
-  case T_CLIST:
-  case T_ECLIST:
-  case T_LCLIST:
-    return (struct f_val) { .type = t, .val.ad = &null_adata };
-
-  default:
-    return (struct f_val) { };
+  switch (t) {
+    case T_PATH:
+    case T_CLIST:
+    case T_ECLIST:
+    case T_LCLIST:
+      return (struct f_val) {
+	.type = t,
+	.val.ad = &null_adata,
+      };
+    default:
+      return (struct f_val) { .type = T_VOID };
   }
 }
 
-
-extern const struct f_val f_const_empty_prefix_set;
-
-enum filter_return f_eval(const struct f_line *expr, struct linpool *tmp_pool, struct f_val *pres);
+enum filter_return f_eval(const struct f_line *expr, struct f_val *pres);
 
 #endif
