@@ -37,12 +37,12 @@ static inline const char *f_instruction_name(enum f_instruction_code fi)
 { return f_instruction_name_(fi) + 3; }
 
 
-int f_const_promotion_(struct f_inst *arg, enum f_type want, int update);
+int f_const_promotion_(struct f_inst *arg, enum btype want, int update);
 
-static inline int f_const_promotion(struct f_inst *arg, enum f_type want)
+static inline int f_const_promotion(struct f_inst *arg, enum btype want)
 { return f_const_promotion_(arg, want, 1); }
 
-static inline int f_try_const_promotion(struct f_inst *arg, enum f_type want)
+static inline int f_try_const_promotion(struct f_inst *arg, enum btype want)
 { return f_const_promotion_(arg, want, 0); }
 
 
@@ -79,10 +79,10 @@ struct filter_iterator {
 
 void f_add_lines(const struct f_line_item *what, struct filter_iterator *fit);
 
-#define FILTER_ITERATE_INIT(fit, filter, pool)			\
+#define FILTER_ITERATE_INIT(fit, line, pool)			\
   ({								\
     BUFFER_INIT((fit)->lines, (pool), 32);			\
-    BUFFER_PUSH((fit)->lines) = (filter)->root;			\
+    BUFFER_PUSH((fit)->lines) = (line);				\
   })
 
 #define FILTER_ITERATE(fit, fi) ({				\
@@ -107,31 +107,15 @@ void f_add_lines(const struct f_line_item *what, struct filter_iterator *fit);
 
 struct filter *f_new_where(struct f_inst *);
 struct f_inst *f_dispatch_method(struct symbol *sym, struct f_inst *obj, struct f_inst *args, int skip);
-struct f_inst *f_dispatch_method_x(const char *name, enum f_type t, struct f_inst *obj, struct f_inst *args);
+struct f_inst *f_dispatch_method_x(const char *name, enum btype t, struct f_inst *obj, struct f_inst *args);
 struct f_inst *f_for_cycle(struct symbol *var, struct f_inst *term, struct f_inst *block);
+struct f_inst *f_implicit_roa_check(struct rtable_config *tab);
 struct f_inst *f_print(struct f_inst *vars, int flush, enum filter_return fret);
 
-static inline struct f_dynamic_attr f_new_dynamic_attr(u8 type, enum f_type f_type, uint code) /* Type as core knows it, type as filters know it, and code of dynamic attribute */
-{ return (struct f_dynamic_attr) { .type = type, .f_type = f_type, .ea_code = code }; }   /* f_type currently unused; will be handy for static type checking */
-static inline struct f_dynamic_attr f_new_dynamic_attr_bit(u8 bit, enum f_type f_type, uint code) /* Type as core knows it, type as filters know it, and code of dynamic attribute */
-{ return (struct f_dynamic_attr) { .type = EAF_TYPE_BITFIELD, .bit = bit, .f_type = f_type, .ea_code = code }; }   /* f_type currently unused; will be handy for static type checking */
-static inline struct f_static_attr f_new_static_attr(int f_type, int code, int readonly)
-{ return (struct f_static_attr) { .f_type = f_type, .sa_code = code, .readonly = readonly }; }
+static inline struct f_static_attr f_new_static_attr(btype type, int code, int readonly)
+{ return (struct f_static_attr) { .type = type, .sa_code = code, .readonly = readonly }; }
+struct f_inst *f_generate_roa_check(struct rtable_config *table, struct f_inst *prefix, struct f_inst *asn);
 
-static inline int f_type_attr(int f_type) {
-  switch (f_type) {
-    case T_INT:		return EAF_TYPE_INT;
-    case T_IP:		return EAF_TYPE_IP_ADDRESS;
-    case T_QUAD:	return EAF_TYPE_ROUTER_ID;
-    case T_PATH:	return EAF_TYPE_AS_PATH;
-    case T_CLIST:	return EAF_TYPE_INT_SET;
-    case T_ECLIST:	return EAF_TYPE_EC_SET;
-    case T_LCLIST:	return EAF_TYPE_LC_SET;
-    case T_BYTESTRING:	return EAF_TYPE_OPAQUE;
-    default:
-      cf_error("Custom route attribute of unsupported type");
-  }
-}
 
 /* Hook for call bt_assert() function in configuration */
 extern void (*bt_assert_hook)(int result, const struct f_line_item *assert);
