@@ -31,7 +31,7 @@ t_reconfig(const void *arg)
     return 0;
 
   struct symbol *s;
-  WALK_LIST(s, config->symbols)
+  WALK_LIST(s, OBSREF_GET(config)->symbols)
     if ((s->class == SYM_FUNCTION) || (s->class == SYM_FILTER))
       bt_assert_msg((s->flags & SYM_FLAG_SAME), "Symbol %s same check", s->name);
 
@@ -46,7 +46,7 @@ run_function(const void *arg)
   if (t->cmp)
     return t->result == f_same(t->fn, t->cmp);
 
-  enum filter_return fret = f_eval(t->fn, tmp_linpool, NULL);
+  enum filter_return fret = f_eval(t->fn, NULL);
 
   return (fret < F_REJECT);
 }
@@ -70,6 +70,7 @@ int
 main(int argc, char *argv[])
 {
   bt_init(argc, argv);
+
   bt_bird_init();
 
   bt_assert_hook = bt_assert_filter;
@@ -78,14 +79,16 @@ main(int argc, char *argv[])
   if (!bt_config_file_parse(BT_CONFIG_FILE))
     abort();
 
-  bt_test_suite_arg(t_reconfig, BT_CONFIG_FILE ".overlay", "Testing reconfiguration to overlay");
-  bt_test_suite_arg(t_reconfig, BT_CONFIG_FILE, "Testing reconfiguration back");
-  bt_test_suite_arg(t_reconfig, BT_CONFIG_FILE, "Testing reconfiguration to the same file");
+  bt_test_suite_arg_extra(t_reconfig, BT_CONFIG_FILE ".overlay", 0, BT_TIMEOUT, "Testing reconfiguration to overlay");
+  bt_test_suite_arg_extra(t_reconfig, BT_CONFIG_FILE, 0, BT_TIMEOUT, "Testing reconfiguration back");
+  bt_test_suite_arg_extra(t_reconfig, BT_CONFIG_FILE, 0, BT_TIMEOUT, "Testing reconfiguration to the same file");
+
+  /* Hack: this should be enough to let the other thread run and make the ROA table converge */
+  birdloop_yield();
 
   struct f_bt_test_suite *t;
-  WALK_LIST(t, config->tests)
-    bt_test_suite_base(run_function, t->fn_name, t, BT_FORKING, BT_TIMEOUT, "%s", t->dsc);
+  WALK_LIST(t, OBSREF_GET(config)->tests)
+    bt_test_suite_base(run_function, t->fn_name, t, 0, BT_TIMEOUT, "%s", t->dsc);
 
-  bt_bird_cleanup();
   return bt_exit_value();
 }
