@@ -156,6 +156,131 @@ ip6_classify(ip6_addr *a)
 }
 
 
+/*
+ *	IPv6 bit shifting
+ */
+
+ip6_addr
+ip6_shift_left(ip6_addr a, uint bits)
+{
+  if (bits == 0)
+    return a;
+
+  if (bits > 127)
+    return IP6_NONE;
+
+  int words = bits / 32;
+  int rem = bits % 32;
+
+  for (int i = 0; i < 3 - words; i++)
+    a.addr[i] = (a.addr[i + words] << rem) |
+      (rem ? (a.addr[i + words + 1] >> (32 - rem)) : 0);
+
+  a.addr[3 - words] = a.addr[3] << rem;
+  memset(&a.addr[4 - words], 0, words * 4);
+
+  return a;
+}
+
+ip6_addr
+ip6_shift_right(ip6_addr a, uint bits)
+{
+  if (bits == 0)
+    return a;
+
+  if (bits > 127)
+    return IP6_NONE;
+
+  int words = bits / 32;
+  int rem = bits % 32;
+
+  for (int i = 3; i > words; i--)
+    a.addr[i] = (a.addr[i - words] >> rem) |
+      (rem ? (a.addr[i - words - 1] << (32 - rem)) : 0);
+
+  a.addr[words] = a.addr[0] >> rem;
+  memset(&a.addr[0], 0, words * 4);
+
+  return a;
+}
+
+
+static const char numbers[] = {
+  '0',   0,   0, 0,  '1',   0,   0, 0,  '2',   0,   0, 0,  '3',   0,   0, 0,  '4',   0,   0, 0,  '5',   0,   0, 0,  '6',   0,   0, 0,  '7',   0,   0, 0,
+  '8',   0,   0, 0,  '9',   0,   0, 0,  '1', '0',   0, 0,  '1', '1',   0, 0,  '1', '2',   0, 0,  '1', '3',   0, 0,  '1', '4',   0, 0,  '1', '5',   0, 0,
+  '1', '6',   0, 0,  '1', '7',   0, 0,  '1', '8',   0, 0,  '1', '9',   0, 0,  '2', '0',   0, 0,  '2', '1',   0, 0,  '2', '2',   0, 0,  '2', '3',   0, 0,
+  '2', '4',   0, 0,  '2', '5',   0, 0,  '2', '6',   0, 0,  '2', '7',   0, 0,  '2', '8',   0, 0,  '2', '9',   0, 0,  '3', '0',   0, 0,  '3', '1',   0, 0,
+  '3', '2',   0, 0,  '3', '3',   0, 0,  '3', '4',   0, 0,  '3', '5',   0, 0,  '3', '6',   0, 0,  '3', '7',   0, 0,  '3', '8',   0, 0,  '3', '9',   0, 0,
+  '4', '0',   0, 0,  '4', '1',   0, 0,  '4', '2',   0, 0,  '4', '3',   0, 0,  '4', '4',   0, 0,  '4', '5',   0, 0,  '4', '6',   0, 0,  '4', '7',   0, 0,
+  '4', '8',   0, 0,  '4', '9',   0, 0,  '5', '0',   0, 0,  '5', '1',   0, 0,  '5', '2',   0, 0,  '5', '3',   0, 0,  '5', '4',   0, 0,  '5', '5',   0, 0,
+  '5', '6',   0, 0,  '5', '7',   0, 0,  '5', '8',   0, 0,  '5', '9',   0, 0,  '6', '0',   0, 0,  '6', '1',   0, 0,  '6', '2',   0, 0,  '6', '3',   0, 0,
+  '6', '4',   0, 0,  '6', '5',   0, 0,  '6', '6',   0, 0,  '6', '7',   0, 0,  '6', '8',   0, 0,  '6', '9',   0, 0,  '7', '0',   0, 0,  '7', '1',   0, 0,
+  '7', '2',   0, 0,  '7', '3',   0, 0,  '7', '4',   0, 0,  '7', '5',   0, 0,  '7', '6',   0, 0,  '7', '7',   0, 0,  '7', '8',   0, 0,  '7', '9',   0, 0,
+  '8', '0',   0, 0,  '8', '1',   0, 0,  '8', '2',   0, 0,  '8', '3',   0, 0,  '8', '4',   0, 0,  '8', '5',   0, 0,  '8', '6',   0, 0,  '8', '7',   0, 0,
+  '8', '8',   0, 0,  '8', '9',   0, 0,  '9', '0',   0, 0,  '9', '1',   0, 0,  '9', '2',   0, 0,  '9', '3',   0, 0,  '9', '4',   0, 0,  '9', '5',   0, 0,
+  '9', '6',   0, 0,  '9', '7',   0, 0,  '9', '8',   0, 0,  '9', '9',   0, 0,  '1', '0', '0', 0,  '1', '0', '1', 0,  '1', '0', '2', 0,  '1', '0', '3', 0,
+  '1', '0', '4', 0,  '1', '0', '5', 0,  '1', '0', '6', 0,  '1', '0', '7', 0,  '1', '0', '8', 0,  '1', '0', '9', 0,  '1', '1', '0', 0,  '1', '1', '1', 0,
+  '1', '1', '2', 0,  '1', '1', '3', 0,  '1', '1', '4', 0,  '1', '1', '5', 0,  '1', '1', '6', 0,  '1', '1', '7', 0,  '1', '1', '8', 0,  '1', '1', '9', 0,
+  '1', '2', '0', 0,  '1', '2', '1', 0,  '1', '2', '2', 0,  '1', '2', '3', 0,  '1', '2', '4', 0,  '1', '2', '5', 0,  '1', '2', '6', 0,  '1', '2', '7', 0,
+  '1', '2', '8', 0,  '1', '2', '9', 0,  '1', '3', '0', 0,  '1', '3', '1', 0,  '1', '3', '2', 0,  '1', '3', '3', 0,  '1', '3', '4', 0,  '1', '3', '5', 0,
+  '1', '3', '6', 0,  '1', '3', '7', 0,  '1', '3', '8', 0,  '1', '3', '9', 0,  '1', '4', '0', 0,  '1', '4', '1', 0,  '1', '4', '2', 0,  '1', '4', '3', 0,
+  '1', '4', '4', 0,  '1', '4', '5', 0,  '1', '4', '6', 0,  '1', '4', '7', 0,  '1', '4', '8', 0,  '1', '4', '9', 0,  '1', '5', '0', 0,  '1', '5', '1', 0,
+  '1', '5', '2', 0,  '1', '5', '3', 0,  '1', '5', '4', 0,  '1', '5', '5', 0,  '1', '5', '6', 0,  '1', '5', '7', 0,  '1', '5', '8', 0,  '1', '5', '9', 0,
+  '1', '6', '0', 0,  '1', '6', '1', 0,  '1', '6', '2', 0,  '1', '6', '3', 0,  '1', '6', '4', 0,  '1', '6', '5', 0,  '1', '6', '6', 0,  '1', '6', '7', 0,
+  '1', '6', '8', 0,  '1', '6', '9', 0,  '1', '7', '0', 0,  '1', '7', '1', 0,  '1', '7', '2', 0,  '1', '7', '3', 0,  '1', '7', '4', 0,  '1', '7', '5', 0,
+  '1', '7', '6', 0,  '1', '7', '7', 0,  '1', '7', '8', 0,  '1', '7', '9', 0,  '1', '8', '0', 0,  '1', '8', '1', 0,  '1', '8', '2', 0,  '1', '8', '3', 0,
+  '1', '8', '4', 0,  '1', '8', '5', 0,  '1', '8', '6', 0,  '1', '8', '7', 0,  '1', '8', '8', 0,  '1', '8', '9', 0,  '1', '9', '0', 0,  '1', '9', '1', 0,
+  '1', '9', '2', 0,  '1', '9', '3', 0,  '1', '9', '4', 0,  '1', '9', '5', 0,  '1', '9', '6', 0,  '1', '9', '7', 0,  '1', '9', '8', 0,  '1', '9', '9', 0,
+  '2', '0', '0', 0,  '2', '0', '1', 0,  '2', '0', '2', 0,  '2', '0', '3', 0,  '2', '0', '4', 0,  '2', '0', '5', 0,  '2', '0', '6', 0,  '2', '0', '7', 0,
+  '2', '0', '8', 0,  '2', '0', '9', 0,  '2', '1', '0', 0,  '2', '1', '1', 0,  '2', '1', '2', 0,  '2', '1', '3', 0,  '2', '1', '4', 0,  '2', '1', '5', 0,
+  '2', '1', '6', 0,  '2', '1', '7', 0,  '2', '1', '8', 0,  '2', '1', '9', 0,  '2', '2', '0', 0,  '2', '2', '1', 0,  '2', '2', '2', 0,  '2', '2', '3', 0,
+  '2', '2', '4', 0,  '2', '2', '5', 0,  '2', '2', '6', 0,  '2', '2', '7', 0,  '2', '2', '8', 0,  '2', '2', '9', 0,  '2', '3', '0', 0,  '2', '3', '1', 0,
+  '2', '3', '2', 0,  '2', '3', '3', 0,  '2', '3', '4', 0,  '2', '3', '5', 0,  '2', '3', '6', 0,  '2', '3', '7', 0,  '2', '3', '8', 0,  '2', '3', '9', 0,
+  '2', '4', '0', 0,  '2', '4', '1', 0,  '2', '4', '2', 0,  '2', '4', '3', 0,  '2', '4', '4', 0,  '2', '4', '5', 0,  '2', '4', '6', 0,  '2', '4', '7', 0,
+  '2', '4', '8', 0,  '2', '4', '9', 0,  '2', '5', '0', 0,  '2', '5', '1', 0,  '2', '5', '2', 0,  '2', '5', '3', 0,  '2', '5', '4', 0,  '2', '5', '5', 0,
+};
+
+static inline char *
+print_u8(char *s, u8 n)
+{
+  memcpy(s, numbers + n * 4, 4);
+  return s + 1 + (n >= 10) + (n >= 100);
+}
+
+char *
+ip4_ntop(ip4_addr a, char *b)
+{
+  u32 x = _I(a);
+
+  b = print_u8(b, (x >> 24) & 0xff);
+  *b++ = '.';
+  b = print_u8(b, (x >> 16) & 0xff);
+  *b++ = '.';
+  b = print_u8(b, (x >> 8) & 0xff);
+  *b++ = '.';
+  b = print_u8(b, x & 0xff);
+
+  return b;
+}
+
+char *
+ip4_px_ntop(ip4_addr a, int len, char *b)
+{
+  u32 x = _I(a);
+
+  b = print_u8(b, (x >> 24) & 0xff);
+  *b++ = '.';
+  b = print_u8(b, (x >> 16) & 0xff);
+  *b++ = '.';
+  b = print_u8(b, (x >> 8) & 0xff);
+  *b++ = '.';
+  b = print_u8(b, x & 0xff);
+  *b++ = '/';
+  b = print_u8(b, len & 0xff);
+
+  return b;
+}
+
 
 /*
  *  Conversion of IPv6 address to presentation format and vice versa.
@@ -163,15 +288,6 @@ ip6_classify(ip6_addr *a)
  *  and of course by RFC 2373.
  */
 
-
-char *
-ip4_ntop(ip4_addr a, char *b)
-{
-  u32 x = _I(a);
-  return b + bsprintf(b, "%d.%d.%d.%d", (x >> 24) & 0xff, (x >> 16) & 0xff, (x >> 8) & 0xff, x & 0xff);
-}
-
-
 char *
 ip6_ntop(ip6_addr a, char *b)
 {
