@@ -27,6 +27,7 @@
 
 struct lp_chunk {
   struct lp_chunk *next;
+  struct linpool *lp;
   max_align_t data_align[0];
   byte data[0];
 };
@@ -38,11 +39,11 @@ struct linpool {
   byte *ptr, *end;
   struct lp_chunk *first, *current;		/* Normal (reusable) chunks */
   struct lp_chunk *first_large;			/* Large chunks */
+  struct lp_state *initial;			/* Initial state to restore to */
   uint total, total_large;
 };
 
-_Thread_local linpool *tmp_linpool;
-
+static void *lp_alloc_slow(struct linpool *, uint);
 static void lp_free(resource *);
 static void lp_dump(struct dump_request *, resource *);
 static resource *lp_lookup(resource *, unsigned long);
@@ -67,7 +68,9 @@ static struct resclass lp_class = {
 linpool
 *lp_new(pool *p)
 {
-  return ralloc(p, &lp_class);
+  linpool *m = ralloc(p, &lp_class);
+  m->initial = lp_save(m);
+  return m;
 }
 
 /**
@@ -87,6 +90,7 @@ linpool
 void *
 lp_alloc(linpool *m, uint size)
 {
+  ASSERT_DIE(DG_IS_LOCKED(resource_parent(&m->r)->domain));
   byte *a = (byte *) BIRD_ALIGN((unsigned long) m->ptr, CPU_STRUCT_ALIGN);
   byte *e = a + size;
 
@@ -96,43 +100,46 @@ lp_alloc(linpool *m, uint size)
       return a;
     }
   else
+    return lp_alloc_slow(m, size);
+}
+
+static void *
+lp_alloc_slow(linpool *m, uint size)
     {
       struct lp_chunk *c;
       if (size > LP_DATA_SIZE)
 	{
 	  /* Too large => allocate large chunk */
 	  c = xmalloc(sizeof(struct lp_chunk) + size);
-	  m->total_large += size;
+	  c->lp = m;
 	  c->next = m->first_large;
+
+	  m->total_large += size;
 	  m->first_large = c;
 	}
       else
 	{
-	  if (m->current && m->current->next)
-	    {
-	      /* Still have free chunks from previous incarnation (before lp_flush()) */
-	      c = m->current->next;
-	    }
+	  if (m->current)
+	    ASSERT_DIE(!m->current->next);
+
+	  /* Need to allocate a new chunk */
+	  c = alloc_page();
+
+	  m->total += LP_DATA_SIZE;
+	  c->next = NULL;
+	  c->lp = m;
+
+	  if (m->current)
+	    m->current->next = c;
 	  else
-	    {
-	      /* Need to allocate a new chunk */
-	      c = alloc_page();
+	    m->first = c;
 
-	      m->total += LP_DATA_SIZE;
-	      c->next = NULL;
-
-	      if (m->current)
-		m->current->next = c;
-	      else
-		m->first = c;
-	    }
 	  m->current = c;
 	  m->ptr = c->data + size;
 	  m->end = c->data + LP_DATA_SIZE;
 	}
       return c->data;
     }
-}
 
 /**
  * lp_allocu - allocate unaligned memory from a &linpool
@@ -147,6 +154,7 @@ lp_alloc(linpool *m, uint size)
 void *
 lp_allocu(linpool *m, uint size)
 {
+  ASSERT_DIE(DG_IS_LOCKED(resource_parent(&m->r)->domain));
   byte *a = m->ptr;
   byte *e = a + size;
 
@@ -155,7 +163,7 @@ lp_allocu(linpool *m, uint size)
       m->ptr = e;
       return a;
     }
-  return lp_alloc(m, size);
+  return lp_alloc_slow(m, size);
 }
 
 /**
@@ -185,26 +193,8 @@ lp_allocz(linpool *m, uint size)
 void
 lp_flush(linpool *m)
 {
-  struct lp_chunk *c;
-
-  /* Move ptr to the first chunk and free all other chunks */
-  m->current = c = m->first;
-  m->ptr = c ? c->data : NULL;
-  m->end = c ? c->data + LP_DATA_SIZE : NULL;
-
-  while (c && c->next)
-  {
-    struct lp_chunk *d = c->next;
-    c->next = d->next;
-    free_page(d);
-  }
-
-  while (c = m->first_large)
-    {
-      m->first_large = c->next;
-      xfree(c);
-    }
-  m->total_large = 0;
+  lp_restore(m, m->initial);
+  m->initial = lp_save(m);
 }
 
 /**
@@ -215,13 +205,20 @@ lp_flush(linpool *m)
  * This function saves the state of a linear memory pool. Saved state can be
  * used later to restore the pool (to free memory allocated since).
  */
-void
-lp_save(linpool *m, lp_state *p)
+struct lp_state *
+lp_save(linpool *m)
 {
-  p->current = m->current;
-  p->large = m->first_large;
-  p->total_large = m->total_large;
-  p->ptr = m->ptr;
+  ASSERT_DIE(DG_IS_LOCKED(resource_parent(&m->r)->domain));
+  struct lp_state *p = lp_alloc(m, sizeof(struct lp_state));
+  ASSERT_DIE(m->current);
+  *p = (struct lp_state) {
+    .p = m,
+    .current = m->current,
+    .large = m->first_large,
+    .total_large = m->total_large,
+  };
+
+  return p;
 }
 
 /**
@@ -238,11 +235,14 @@ void
 lp_restore(linpool *m, lp_state *p)
 {
   struct lp_chunk *c;
+  ASSERT_DIE(DG_IS_LOCKED(resource_parent(&m->r)->domain));
 
   /* Move ptr to the saved pos and free all newer large chunks */
-  m->current = c = p->current ?: m->first;
-  m->ptr = p->ptr ?: (c ? c->data : NULL);
-  m->end = c ? (c->data + LP_DATA_SIZE) : NULL;
+  ASSERT_DIE(p->current);
+  ASSERT_DIE(p->p == m);
+  m->current = c = p->current;
+  m->ptr = (byte *) p;
+  m->end = c->data + LP_DATA_SIZE;
   m->total_large = p->total_large;
 
   while ((c = m->first_large) && (c != p->large))
@@ -250,6 +250,12 @@ lp_restore(linpool *m, lp_state *p)
       m->first_large = c->next;
       xfree(c);
     }
+
+  while (c = m->current->next)
+    {
+      m->current->next = c->next;
+      free_page(c);
+    }
 }
 
 static void
