@@ -22,17 +22,19 @@ extern int configuring;
 void
 cmd_show_status(void)
 {
-  struct timeformat *tf_base = this_cli->tf ?: &config->tf_base;
+  rcu_read_lock();
+  struct global_runtime *gr = atomic_load_explicit(&global_runtime, memory_order_acquire);
+  struct timeformat *tf = this_cli->tf ?: &gr->tf_base;
   byte tim[TM_DATETIME_BUFFER_SIZE];
 
   cli_msg(-1000, "BIRD " BIRD_VERSION);
-  tm_format_time(tim, tf_base, current_time());
-  cli_msg(-1011, "Router ID is %R", config->router_id);
-  cli_msg(-1011, "Hostname is %s", config->hostname);
+  tm_format_time(tim, tf, current_time());
+  cli_msg(-1011, "Router ID is %R", gr->router_id);
+  cli_msg(-1011, "Hostname is %s", gr->hostname);
   cli_msg(-1011, "Current server time is %s", tim);
-  tm_format_time(tim, tf_base, boot_time);
+  tm_format_time(tim, tf, boot_time);
   cli_msg(-1011, "Last reboot on %s", tim);
-  tm_format_time(tim, tf_base, config->load_time);
+  tm_format_time(tim, tf, gr->load_time);
   cli_msg(-1011, "Last reconfiguration on %s", tim);
 
   graceful_restart_show_status();
@@ -43,6 +45,8 @@ cmd_show_status(void)
     cli_msg(13, "Reconfiguration in progress");
   else
     cli_msg(13, "Daemon is up and running");
+
+  rcu_read_unlock();
 }
 
 void
@@ -52,12 +56,9 @@ cmd_show_symbols(struct sym_show_data *sd)
     cli_msg(1010, "%-8s\t%s", sd->sym->name, cf_symbol_class_name(sd->sym));
   else
   {
-    for (const struct sym_scope *scope = config->root_scope; scope; scope = scope->next)
+    for (const struct sym_scope *scope = sd->config->root_scope; scope; scope = scope->next)
       HASH_WALK(scope->hash, next, sym)
       {
-	if (!sym->scope->active)
-	  continue;
-
 	if (sd->type && (sym->class != sd->type))
 	  continue;
 
@@ -110,8 +111,6 @@ print_size(char *dsc, struct resmem vals)
 
 extern pool *rt_table_pool;
 extern pool *rta_pool;
-extern uint *pages_kept;
-extern uint pages_kept_cold, pages_kept_cold_index, pages_total;
 
 void
 cmd_show_memory(void)
@@ -124,22 +123,31 @@ cmd_show_memory(void)
   print_size("Current config:", rmemsize(config_pool));
   struct resmem total = rmemsize(&root_pool);
 #ifdef HAVE_MMAP
-  uint pages_standby = *pages_kept + pages_kept_cold_index;
-  print_size("Standby memory:", (struct resmem) { .overhead = page_size * pages_standby });
-  total.overhead += page_size * pages_standby;
+  uint hot_pages = atomic_load_explicit(&pages_kept, memory_order_relaxed)
+		+ atomic_load_explicit(&pages_kept_locally, memory_order_relaxed);
+  uint cold_pages_index = atomic_load_explicit(&pages_kept_cold_index, memory_order_relaxed);
+  print_size("Standby memory:", (struct resmem) { .overhead = page_size * (hot_pages + cold_pages_index) });
+  total.overhead += page_size * (hot_pages + cold_pages_index);
 #endif
+
   print_size("Total:", total);
   cli_msg(-1018, "");
 
-  uint pages_active = pages_total - *pages_kept - pages_kept_cold;
+#ifdef HAVE_MMAP
+  uint cold_pages = atomic_load_explicit(&pages_kept_cold, memory_order_relaxed);
+  uint pages_total_loc = atomic_load_explicit(&pages_total, memory_order_relaxed);
+  uint pages_active = pages_total_loc - hot_pages - cold_pages_index - cold_pages;
+  
   struct size_args active = get_size_args(page_size * pages_active);
-  struct size_args kept = get_size_args(page_size * *pages_kept);
-  struct size_args cold = get_size_args(page_size * pages_kept_cold);
+  struct size_args kept = get_size_args(page_size * (hot_pages + cold_pages_index));
+  struct size_args cold = get_size_args(page_size * cold_pages);
 
   cli_msg(-1018, "%-17s " SIZE_FORMAT, "Active pages:", SIZE_ARGS(active));
   cli_msg(-1018, "%-17s " SIZE_FORMAT, "Kept free pages:", SIZE_ARGS(kept));
   cli_msg(-1018, "%-17s " SIZE_FORMAT, "Cold free pages:", SIZE_ARGS(cold));
 
+#endif
+  cli_msg(-1028, "Hot page cache depleted while in RCU: %d", atomic_load_explicit(&alloc_locking_in_rcu, memory_order_relaxed));
   cli_msg(0, "");
 }
 
@@ -147,9 +155,9 @@ void
 cmd_eval(const struct f_line *expr)
 {
   buffer buf;
-  LOG_BUFFER_INIT(buf);
+  STACK_BUFFER_INIT(buf, CLI_MSG_SIZE);
 
-  if (f_eval_buf(expr, this_cli->parser_pool, &buf) > F_RETURN)
+  if (f_eval_buf(expr, &buf) > F_RETURN)
     {
       cli_msg(8008, "runtime error");
       return;
