@@ -60,6 +60,19 @@
 static slab *neigh_slab;
 static list neigh_hash_table[NEIGH_HASH_SIZE], sticky_neigh_list;
 
+void if_link(struct iface *);
+void if_unlink(struct iface *);
+void ifa_link(struct ifa *);
+void ifa_unlink(struct ifa *);
+
+extern list global_iface_list;
+
+extern DOMAIN(attrs) iface_domain;
+
+#define IFACE_LOCK	LOCK_DOMAIN(attrs, iface_domain)
+#define IFACE_UNLOCK	UNLOCK_DOMAIN(attrs, iface_domain)
+#define IFACE_ASSERT_LOCKED	ASSERT_DIE(DOMAIN_IS_LOCKED(attrs, iface_domain))
+
 static inline uint
 neigh_hash(struct proto *p, ip_addr a, struct iface *i)
 {
@@ -142,7 +155,7 @@ if_connected(ip_addr a, struct iface *i, struct ifa **ap, uint flags)
 }
 
 static inline int
-if_connected_any(ip_addr a, struct iface *vrf, uint vrf_set, struct iface **iface, struct ifa **addr, uint flags)
+if_connected_any(ip_addr a, struct iface *vrf, struct iface **iface, struct ifa **addr, uint flags)
 {
   struct iface *i;
   struct ifa *b;
@@ -152,8 +165,8 @@ if_connected_any(ip_addr a, struct iface *vrf, uint vrf_set, struct iface **ifac
   *addr = NULL;
 
   /* Prefer SCOPE_HOST or longer prefix */
-  WALK_LIST(i, iface_list)
-    if ((!vrf_set || if_in_vrf(i, vrf)) && ((s = if_connected(a, i, &b, flags)) >= 0))
+  WALK_LIST(i, global_iface_list)
+    if ((!vrf || if_in_vrf(i,vrf)) && ((s = if_connected(a, i, &b, flags)) >= 0))
       if (scope_better(s, scope) || (scope_remote(s, scope) && ifa_better(b, *addr)))
       {
 	*iface = i;
@@ -210,6 +223,8 @@ if_intersect(struct iface *ia, struct iface *ib)
 neighbor *
 neigh_find(struct proto *p, ip_addr a, struct iface *iface, uint flags)
 {
+  IFACE_LOCK;
+
   neighbor *n;
   int class, scope = -1;
   uint h = neigh_hash(p, a, iface);
@@ -217,27 +232,31 @@ neigh_find(struct proto *p, ip_addr a, struct iface *iface, uint flags)
   struct ifa *addr = NULL;
 
   WALK_LIST(n, neigh_hash_table[h])	/* Search the cache */
-    if ((n->proto == p) && ipa_equal(n->addr, a) && (n->ifreq == iface))
+    if ((n->proto == p) && ipa_equal(n->addr, a) && (n->ifreq == iface) &&
+	((n->flags & NEF_ONLINK) == (flags & NEF_ONLINK)))
+    {
+      IFACE_UNLOCK;
       return n;
+    }
 
   if (flags & NEF_IFACE)
   {
     if (ipa_nonzero(a) || !iface)
-      return NULL;
+      goto bad;
   }
   else
   {
     class = ipa_classify(a);
     if (class < 0)			/* Invalid address */
-      return NULL;
+      goto bad;
     if (((class & IADDR_SCOPE_MASK) == SCOPE_HOST) ||
 	(((class & IADDR_SCOPE_MASK) == SCOPE_LINK) && !iface) ||
 	!(class & IADDR_HOST))
-      return NULL;			/* Bad scope or a somecast */
+      goto bad;				/* Bad scope or a somecast */
   }
 
   if ((flags & NEF_ONLINK) && !iface)
-      return NULL;
+    goto bad;
 
   if (iface)
   {
@@ -245,26 +264,35 @@ neigh_find(struct proto *p, ip_addr a, struct iface *iface, uint flags)
     iface = (scope < 0) ? NULL : iface;
   }
   else
-    scope = if_connected_any(a, p->vrf, p->vrf_set, &iface, &addr, flags);
+    scope = if_connected_any(a, p->vrf, &iface, &addr, flags);
 
   /* scope < 0 means i don't know neighbor */
   /* scope >= 0  <=>  iface != NULL */
 
   if ((scope < 0) && !(flags & NEF_STICKY))
-    return NULL;
+    goto bad;
 
   n = sl_allocz(neigh_slab);
   add_tail(&neigh_hash_table[h], &n->n);
   add_tail((scope >= 0) ? &iface->neighbors : &sticky_neigh_list, &n->if_n);
+  proto_neigh_add_tail(&p->neighbors, n);
   n->addr = a;
-  n->ifa = addr;
-  n->iface = iface;
-  n->ifreq = ifreq;
+  ifa_link(n->ifa = addr);
+  if_link(n->iface = iface);
+  if_link(n->ifreq = ifreq);
   n->proto = p;
   n->flags = flags;
   n->scope = scope;
 
+  neigh_link_locked(n);
+  neigh_unlink_later(n);
+
+  IFACE_UNLOCK;
   return n;
+
+bad:
+  IFACE_UNLOCK;
+  return NULL;
 }
 
 /**
@@ -273,18 +301,18 @@ neigh_find(struct proto *p, ip_addr a, struct iface *iface, uint flags)
  *
  * This functions dumps the contents of a given neighbor entry to debug output.
  */
-void
-neigh_dump(neighbor *n)
+static void
+neigh_dump(struct dump_request *dreq, neighbor *n)
 {
-  debug("%p %I %s %s ", n, n->addr,
+  RDUMP("%p %I %s %s ", n, n->addr,
 	n->iface ? n->iface->name : "[]",
 	n->ifreq ? n->ifreq->name : "[]");
-  debug("%s %p %08x scope %s", n->proto->name, n->data, n->aux, ip_scope_text(n->scope));
+  RDUMP("%s %p %08x scope %s", n->proto->name, n->data, n->aux, ip_scope_text(n->scope));
   if (n->flags & NEF_STICKY)
-    debug(" STICKY");
+    RDUMP(" STICKY");
   if (n->flags & NEF_ONLINK)
-    debug(" ONLINK");
-  debug("\n");
+    RDUMP(" ONLINK");
+  RDUMP("\n");
 }
 
 /**
@@ -293,34 +321,40 @@ neigh_dump(neighbor *n)
  * This function dumps the contents of the neighbor cache to debug output.
  */
 void
-neigh_dump_all(void)
+neigh_dump_all(struct dump_request *dreq)
 {
+  IFACE_LOCK;
+
   neighbor *n;
   int i;
 
-  debug("Known neighbors:\n");
+  RDUMP("Known neighbors:\n");
   for(i=0; i<NEIGH_HASH_SIZE; i++)
     WALK_LIST(n, neigh_hash_table[i])
-      neigh_dump(n);
-  debug("\n");
+      neigh_dump(dreq, n);
+  RDUMP("\n");
+
+  IFACE_UNLOCK;
 }
 
 static inline void
 neigh_notify(neighbor *n)
 {
-  if (n->proto->neigh_notify && (n->proto->proto_state != PS_STOP))
-    n->proto->neigh_notify(n);
+  IFACE_ASSERT_LOCKED;
+  if_enqueue_notify_to((struct iface_notification) { .type = IFNOT_NEIGHBOR, .n = n, }, &n->proto->iface_sub);
 }
 
 static void
 neigh_up(neighbor *n, struct iface *i, struct ifa *a, int scope)
 {
   DBG("Waking up sticky neighbor %I\n", n->addr);
-  n->iface = i;
-  n->ifa = a;
+  if_link(n->iface = i);
+  ifa_link(n->ifa = a);
+
   n->scope = scope;
 
-  rem_node(&n->if_n);
+  rem_node(&n->if_n); /* HACK: Here the neighbor is always in the sticky list,
+			 regardless whether it is sticky or not */
   add_tail(&i->neighbors, &n->if_n);
 
   neigh_notify(n);
@@ -330,24 +364,74 @@ static void
 neigh_down(neighbor *n)
 {
   DBG("Flushing neighbor %I on %s\n", n->addr, n->iface->name);
-  n->iface = NULL;
-  n->ifa = NULL;
+
   n->scope = -1;
 
   rem_node(&n->if_n);
   add_tail(&sticky_neigh_list, &n->if_n);
 
+  ifa_unlink(n->ifa);
+  n->ifa = NULL;
+
+  if_unlink(n->iface);
+  n->iface = NULL;
+
   neigh_notify(n);
 }
 
-static inline void
-neigh_free(neighbor *n)
+void
+neigh_link_locked(neighbor *n)
 {
+  IFACE_ASSERT_LOCKED;
+  n->uc++;
+}
+
+void
+neigh_unlink_locked(neighbor *n)
+{
+  IFACE_ASSERT_LOCKED;
+  if (--n->uc)
+    return;
+
+  struct proto *p = n->proto;
+  proto_neigh_rem_node(&p->neighbors, n);
+
+  if ((p->proto_state == PS_FLUSH) && EMPTY_TLIST(proto_neigh, &p->neighbors))
+    proto_send_event(p, p->event);
+
+  n->proto = NULL;
+
   rem_node(&n->n);
   rem_node(&n->if_n);
+
+  ifa_unlink(n->ifa);
+  if_unlink(n->iface);
+  if_unlink(n->ifreq);
+
   sl_free(n);
 }
 
+void
+neigh_link(neighbor *n)
+{
+  IFACE_LOCK;
+  neigh_link_locked(n);
+  IFACE_UNLOCK;
+}
+
+void
+neigh_unlink(neighbor *n)
+{
+  IFACE_LOCK;
+  neigh_unlink_locked(n);
+  IFACE_UNLOCK;
+}
+
+void neigh_unlink_deferred(struct deferred_call *dc)
+{
+  neigh_unlink(SKIP_BACK(struct neigh_unlink_deferred, dc, dc)->n);
+}
+
 /**
  * neigh_update: update neighbor entry w.r.t. change on specific iface
  * @n: neighbor to update
@@ -360,6 +444,8 @@ neigh_free(neighbor *n)
 void
 neigh_update(neighbor *n, struct iface *iface)
 {
+  IFACE_ASSERT_LOCKED;
+
   struct proto *p = n->proto;
   struct ifa *ifa = NULL;
   int scope = -1;
@@ -369,7 +455,7 @@ neigh_update(neighbor *n, struct iface *iface)
     return;
 
   /* VRF-bound neighbors ignore changes in other VRFs */
-  if (p->vrf_set && !if_in_vrf(iface, p->vrf))
+  if (p->vrf && !if_in_vrf(iface, p->vrf))
     return;
 
   scope = if_connected(n->addr, iface, &ifa, n->flags);
@@ -379,7 +465,7 @@ neigh_update(neighbor *n, struct iface *iface)
   {
     /* When neighbor is going down, try to respawn it on other ifaces */
     if ((scope < 0) && (n->scope >= 0) && !n->ifreq && (n->flags & NEF_STICKY))
-      scope = if_connected_any(n->addr, p->vrf, p->vrf_set, &iface, &ifa, n->flags);
+      scope = if_connected_any(n->addr, p->vrf, &iface, &ifa, n->flags);
   }
   else
   {
@@ -394,7 +480,8 @@ neigh_update(neighbor *n, struct iface *iface)
   {
     if (ifa != n->ifa)
     {
-      n->ifa = ifa;
+      ifa_unlink(n->ifa);
+      ifa_link(n->ifa = ifa);
       neigh_notify(n);
     }
 
@@ -408,7 +495,7 @@ neigh_update(neighbor *n, struct iface *iface)
 
   if ((n->scope < 0) && !(n->flags & NEF_STICKY))
   {
-    neigh_free(n);
+    neigh_unlink_locked(n);
     return;
   }
 
@@ -429,12 +516,13 @@ neigh_update(neighbor *n, struct iface *iface)
 void
 neigh_if_up(struct iface *i)
 {
+  IFACE_ASSERT_LOCKED;
   struct iface *ii;
   neighbor *n;
   node *x, *y;
 
   /* Update neighbors that might be better off with the new iface */
-  WALK_LIST(ii, iface_list)
+  WALK_LIST(ii, global_iface_list)
     if (!EMPTY_LIST(ii->neighbors) && (ii != i) && if_intersect(i, ii))
       WALK_LIST2_DELSAFE(n, x, y, ii->neighbors, if_n)
 	neigh_update(n, i);
@@ -454,6 +542,7 @@ neigh_if_up(struct iface *i)
 void
 neigh_if_down(struct iface *i)
 {
+  IFACE_ASSERT_LOCKED;
   neighbor *n;
   node *x, *y;
 
@@ -471,6 +560,7 @@ neigh_if_down(struct iface *i)
 void
 neigh_if_link(struct iface *i)
 {
+  IFACE_ASSERT_LOCKED;
   neighbor *n;
   node *x, *y;
 
@@ -491,12 +581,13 @@ neigh_if_link(struct iface *i)
 void
 neigh_ifa_up(struct ifa *a)
 {
+  IFACE_ASSERT_LOCKED;
   struct iface *i = a->iface, *ii;
   neighbor *n;
   node *x, *y;
 
   /* Update neighbors that might be better off with the new ifa */
-  WALK_LIST(ii, iface_list)
+  WALK_LIST(ii, global_iface_list)
     if (!EMPTY_LIST(ii->neighbors) && ifa_intersect(a, ii))
       WALK_LIST2_DELSAFE(n, x, y, ii->neighbors, if_n)
 	neigh_update(n, i);
@@ -509,6 +600,7 @@ neigh_ifa_up(struct ifa *a)
 void
 neigh_ifa_down(struct ifa *a)
 {
+  IFACE_ASSERT_LOCKED;
   struct iface *i = a->iface;
   neighbor *n;
   node *x, *y;
@@ -519,35 +611,6 @@ neigh_ifa_down(struct ifa *a)
       neigh_update(n, i);
 }
 
-static inline void
-neigh_prune_one(neighbor *n)
-{
-  if (n->proto->proto_state != PS_DOWN)
-    return;
-
-  neigh_free(n);
-}
-
-/**
- * neigh_prune - prune neighbor cache
- *
- * neigh_prune() examines all neighbor entries cached and removes those
- * corresponding to inactive protocols. It's called whenever a protocol
- * is shut down to get rid of all its heritage.
- */
-void
-neigh_prune(void)
-{
-  neighbor *n;
-  node *m;
-  int i;
-
-  DBG("Pruning neighbors\n");
-  for(i=0; i<NEIGH_HASH_SIZE; i++)
-    WALK_LIST_DELSAFE(n, m, neigh_hash_table[i])
-      neigh_prune_one(n);
-}
-
 /**
  * neigh_init - initialize the neighbor cache.
  * @if_pool: resource pool to be used for neighbor entries.
