@@ -13,6 +13,15 @@ struct rt_dev_config {
   struct proto_config c;
   list iface_list;		/* list of struct iface_patt */
   int check_link;
+
+  struct channel_config *ip4_channel;
+  struct channel_config *ip6_channel;
+};
+
+struct rt_dev_proto {
+  struct proto p;
+  struct channel *ip4_channel;
+  struct channel *ip6_channel;
 };
 
 #endif
