@@ -49,6 +49,10 @@ pipe_rt_notify(struct proto *P, struct channel *src_ch, const net_addr *n, rte *
   struct pipe_proto *p = (void *) P;
   struct channel *dst = (src_ch == p->pri) ? p->sec : p->pri;
 
+  /* Reject everything on shutdown */
+  if (SHUTTING_DOWN)
+    return;
+
   if (!new && !old)
     return;
 
@@ -70,6 +74,10 @@ pipe_preexport(struct channel *C, rte *e)
 {
   struct pipe_proto *p = (void *) C->proto;
 
+  /* Reject everything on shutdown */
+  if (SHUTTING_DOWN)
+    return -1;
+
   /* Avoid direct loopbacks */
   if (e->sender == C->in_req.hook)
     return -1;
@@ -278,7 +286,7 @@ pipe_show_proto_info(struct proto *P)
   channel_show_limit(&p->sec->in_limit, "Export limit:",
       (p->sec->limit_active & (1 << PLD_IN)), p->sec->limit_actions[PLD_IN]);
 
-  if (P->proto_state != PS_DOWN_XX)
+  if (P->proto_state == PS_UP)
     pipe_show_stats(p);
 }
 
